@@ -2,36 +2,31 @@
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
-
 """Unit tests for Bio.SeqIO.index(...) and index_db() functions."""
 
 try:
     import sqlite3
 except ImportError:
-    # Try to run what tests we can on Jython
-    # where we don't expect this to be installed.
+    # Try to run what tests we can in case sqlite3 was not installed
     sqlite3 = None
 
+import gzip
 import os
-import unittest
 import tempfile
 import threading
-import gzip
+import unittest
 import warnings
 from io import BytesIO
 from io import StringIO
+from pathlib import Path
 
-from Bio.SeqRecord import SeqRecord
-from Bio import SeqIO
-from Bio.SeqIO._index import _FormatToRandomAccess
-from Bio.Alphabet import generic_protein, generic_nucleotide, generic_dna
-
-from Bio import BiopythonParserWarning
-from Bio import MissingPythonDependencyError
-
-from seq_tests_common import compare_record
+from seq_tests_common import SeqRecordTestBaseClass
 from test_SeqIO import SeqIOTestBaseClass
 
+from Bio import BiopythonParserWarning
+from Bio import SeqIO
+from Bio.SeqIO._index import _FormatToRandomAccess
+from Bio.SeqRecord import SeqRecord
 
 CUR_DIR = os.getcwd()
 
@@ -86,6 +81,11 @@ if sqlite3:
             self.assertEqual(54, len(d))
             self.assertRaises(FileNotFoundError, d.get_raw, "alpha")
 
+        def test_pathobj(self):
+            """Load existing index from a pathlib.Path object."""
+            d = SeqIO.index_db(Path("Roche/triple_sff.idx"))
+            self.assertEqual(54, len(d))
+
         def test_old_check_same_thread(self):
             """Setting check_same_thread to False doesn't raise an exception."""
             d = SeqIO.index_db("Roche/triple_sff_rel_paths.idx")
@@ -111,13 +111,13 @@ if sqlite3:
         def test_old_contents(self):
             """Check actual filenames in existing indexes."""
             filenames, flag = raw_filenames("Roche/triple_sff.idx")
-            self.assertEqual(flag, None)
+            self.assertIsNone(flag)
             self.assertEqual(
                 filenames, ["E3MFGYR02_no_manifest.sff", "greek.sff", "paired.sff"]
             )
 
             filenames, flag = raw_filenames("Roche/triple_sff_rel_paths.idx")
-            self.assertEqual(flag, True)
+            self.assertTrue(flag)
             self.assertEqual(
                 filenames, ["E3MFGYR02_no_manifest.sff", "greek.sff", "paired.sff"]
             )
@@ -211,7 +211,7 @@ if sqlite3:
 
             # Now directly check the filenames inside the SQLite index:
             filenames, flag = raw_filenames(index_file)
-            self.assertEqual(flag, True)
+            self.assertTrue(flag)
             self.assertEqual(filenames, expt_sff_files)
 
             # Load index...
@@ -225,7 +225,7 @@ if sqlite3:
 
         def test_child_folder_rel(self):
             """Check relative links to child folder."""
-            # Note we expect relative paths recorded with Unix slashs!
+            # Note we expect relative paths recorded with Unix slashes!
             expt_sff_files = [
                 "Roche/E3MFGYR02_no_manifest.sff",
                 "Roche/greek.sff",
@@ -333,67 +333,66 @@ if sqlite3:
             )
 
 
-class IndexDictTests(SeqIOTestBaseClass):
-
+class IndexDictTests(SeqRecordTestBaseClass, SeqIOTestBaseClass):
     tests = [
-        ("Ace/contig1.ace", "ace", generic_dna),
-        ("Ace/consed_sample.ace", "ace", None),
-        ("Ace/seq.cap.ace", "ace", generic_dna),
-        ("Quality/wrapping_original_sanger.fastq", "fastq", None),
-        ("Quality/example.fastq", "fastq", None),  # Unix newlines
-        ("Quality/example.fastq", "fastq-sanger", generic_dna),
-        ("Quality/example_dos.fastq", "fastq", None),  # DOS/Windows newlines
-        ("Quality/tricky.fastq", "fastq", generic_nucleotide),
-        ("Quality/sanger_faked.fastq", "fastq-sanger", generic_dna),
-        ("Quality/solexa_faked.fastq", "fastq-solexa", generic_dna),
-        ("Quality/illumina_faked.fastq", "fastq-illumina", generic_dna),
-        ("Quality/zero_length.fastq", "fastq", generic_dna),
-        ("EMBL/epo_prt_selection.embl", "embl", None),
-        ("EMBL/U87107.embl", "embl", None),
-        ("EMBL/TRBG361.embl", "embl", None),
-        ("EMBL/kipo_prt_sample.embl", "embl", None),
-        ("EMBL/A04195.imgt", "embl", None),  # Not a proper EMBL file, an IMGT file
-        ("EMBL/A04195.imgt", "imgt", None),
-        ("EMBL/hla_3260_sample.imgt", "imgt", None),
-        ("EMBL/patents.embl", "embl", generic_protein),
-        ("EMBL/AAA03323.embl", "embl", None),
-        ("GenBank/NC_000932.faa", "fasta", generic_protein),
-        ("GenBank/NC_005816.faa", "fasta", generic_protein),
-        ("GenBank/NC_005816.tsv", "tab", generic_protein),
-        ("GenBank/NC_005816.ffn", "fasta", generic_dna),
-        ("GenBank/NC_005816.fna", "fasta", generic_dna),
-        ("GenBank/NC_005816.gb", "gb", None),
-        ("GenBank/cor6_6.gb", "genbank", None),
-        ("GenBank/empty_accession.gbk", "gb", None),
-        ("GenBank/empty_version.gbk", "gb", None),
-        ("IntelliGenetics/vpu_nucaligned.txt", "ig", generic_nucleotide),
-        ("IntelliGenetics/TAT_mase_nuc.txt", "ig", None),
-        ("IntelliGenetics/VIF_mase-pro.txt", "ig", generic_protein),
-        ("Phd/phd1", "phd", generic_dna),
-        ("Phd/phd2", "phd", None),
-        ("Phd/phd_solexa", "phd", generic_dna),
-        ("Phd/phd_454", "phd", generic_dna),
-        ("NBRF/B_nuc.pir", "pir", generic_nucleotide),
-        ("NBRF/Cw_prot.pir", "pir", generic_protein),
-        ("NBRF/clustalw.pir", "pir", None),
-        ("SwissProt/sp001", "swiss", None),
-        ("SwissProt/sp010", "swiss", None),
-        ("SwissProt/sp016", "swiss", None),
-        ("SwissProt/multi_ex.txt", "swiss", None),
-        ("SwissProt/multi_ex.xml", "uniprot-xml", None),
-        ("SwissProt/multi_ex.fasta", "fasta", None),
-        ("Roche/E3MFGYR02_random_10_reads.sff", "sff", generic_dna),
-        ("Roche/E3MFGYR02_random_10_reads.sff", "sff-trim", generic_dna),
-        ("Roche/E3MFGYR02_index_at_start.sff", "sff", generic_dna),
-        ("Roche/E3MFGYR02_index_in_middle.sff", "sff", generic_dna),
-        ("Roche/E3MFGYR02_alt_index_at_start.sff", "sff", generic_dna),
-        ("Roche/E3MFGYR02_alt_index_in_middle.sff", "sff", generic_dna),
-        ("Roche/E3MFGYR02_alt_index_at_end.sff", "sff", generic_dna),
-        ("Roche/E3MFGYR02_no_manifest.sff", "sff", generic_dna),
-        ("Roche/greek.sff", "sff", generic_nucleotide),
-        ("Roche/greek.sff", "sff-trim", generic_nucleotide),
-        ("Roche/paired.sff", "sff", None),
-        ("Roche/paired.sff", "sff-trim", None),
+        ("Ace/contig1.ace", "ace"),
+        ("Ace/consed_sample.ace", "ace"),
+        ("Ace/seq.cap.ace", "ace"),
+        ("Quality/wrapping_original_sanger.fastq", "fastq"),
+        ("Quality/example.fastq", "fastq"),  # Unix newlines
+        ("Quality/example.fastq", "fastq-sanger"),
+        ("Quality/example_dos.fastq", "fastq"),  # DOS/Windows newlines
+        ("Quality/tricky.fastq", "fastq"),
+        ("Quality/sanger_faked.fastq", "fastq-sanger"),
+        ("Quality/solexa_faked.fastq", "fastq-solexa"),
+        ("Quality/illumina_faked.fastq", "fastq-illumina"),
+        ("Quality/zero_length.fastq", "fastq"),
+        ("EMBL/epo_prt_selection.embl", "embl"),
+        ("EMBL/U87107.embl", "embl"),
+        ("EMBL/TRBG361.embl", "embl"),
+        ("EMBL/kipo_prt_sample.embl", "embl"),
+        ("EMBL/A04195.imgt", "embl"),  # Not a proper EMBL file, an IMGT file
+        ("EMBL/A04195.imgt", "imgt"),
+        ("EMBL/hla_3260_sample.imgt", "imgt"),
+        ("EMBL/patents.embl", "embl"),
+        ("EMBL/AAA03323.embl", "embl"),
+        ("GenBank/NC_000932.faa", "fasta"),
+        ("GenBank/NC_005816.faa", "fasta"),
+        ("GenBank/NC_005816.tsv", "tab"),
+        ("GenBank/NC_005816.ffn", "fasta"),
+        ("GenBank/NC_005816.fna", "fasta"),
+        ("GenBank/NC_005816.gb", "gb"),
+        ("GenBank/cor6_6.gb", "genbank"),
+        ("GenBank/empty_accession.gbk", "gb"),
+        ("GenBank/empty_version.gbk", "gb"),
+        ("IntelliGenetics/vpu_nucaligned.txt", "ig"),
+        ("IntelliGenetics/TAT_mase_nuc.txt", "ig"),
+        ("IntelliGenetics/VIF_mase-pro.txt", "ig"),
+        ("Phd/phd1", "phd"),
+        ("Phd/phd2", "phd"),
+        ("Phd/phd_solexa", "phd"),
+        ("Phd/phd_454", "phd"),
+        ("NBRF/B_nuc.pir", "pir"),
+        ("NBRF/Cw_prot.pir", "pir"),
+        ("NBRF/clustalw.pir", "pir"),
+        ("SwissProt/Q13454.txt", "swiss"),
+        ("SwissProt/Q13639.txt", "swiss"),
+        ("SwissProt/sp016", "swiss"),
+        ("SwissProt/multi_ex.txt", "swiss"),
+        ("SwissProt/multi_ex.xml", "uniprot-xml"),
+        ("SwissProt/multi_ex.fasta", "fasta"),
+        ("Roche/E3MFGYR02_random_10_reads.sff", "sff"),
+        ("Roche/E3MFGYR02_random_10_reads.sff", "sff-trim"),
+        ("Roche/E3MFGYR02_index_at_start.sff", "sff"),
+        ("Roche/E3MFGYR02_index_in_middle.sff", "sff"),
+        ("Roche/E3MFGYR02_alt_index_at_start.sff", "sff"),
+        ("Roche/E3MFGYR02_alt_index_in_middle.sff", "sff"),
+        ("Roche/E3MFGYR02_alt_index_at_end.sff", "sff"),
+        ("Roche/E3MFGYR02_no_manifest.sff", "sff"),
+        ("Roche/greek.sff", "sff"),
+        ("Roche/greek.sff", "sff-trim"),
+        ("Roche/paired.sff", "sff"),
+        ("Roche/paired.sff", "sff-trim"),
     ]
 
     def setUp(self):
@@ -407,7 +406,7 @@ class IndexDictTests(SeqIOTestBaseClass):
             os.remove(self.index_tmp)
 
     def check_dict_methods(self, rec_dict, keys, ids, msg):
-        self.assertEqual(set(keys), set(rec_dict), msg=msg)
+        self.assertCountEqual(keys, rec_dict.keys(), msg=msg)
         # This is redundant, I just want to make sure len works:
         self.assertEqual(len(keys), len(rec_dict), msg=msg)
         # Make sure boolean evaluation works
@@ -416,11 +415,11 @@ class IndexDictTests(SeqIOTestBaseClass):
             self.assertIn(key, rec_dict, msg=msg)
             self.assertEqual(id, rec_dict[key].id, msg=msg)
             self.assertEqual(id, rec_dict.get(key).id, msg=msg)
-        # Check non-existant keys,
+        # Check non-existent keys,
         assert chr(0) not in keys, "Bad example in test"
         with self.assertRaises(KeyError, msg=msg):
             rec = rec_dict[chr(0)]
-        self.assertEqual(rec_dict.get(chr(0)), None, msg=msg)
+        self.assertIsNone(rec_dict.get(chr(0)), msg=msg)
         self.assertEqual(rec_dict.get(chr(0), chr(1)), chr(1), msg=msg)
         with self.assertRaises(AttributeError, msg=msg):
             rec_dict.iteritems
@@ -433,24 +432,24 @@ class IndexDictTests(SeqIOTestBaseClass):
             self.assertIsInstance(rec, SeqRecord, msg=msg)
             self.assertIn(rec.id, ids, msg=msg)
 
-    def simple_check(self, filename, fmt, alphabet, comp):
+    def simple_check(self, filename, fmt, comp):
         """Check indexing (without a key function)."""
-        msg = "Test failure parsing file %s with format %s" % (filename, fmt)
+        msg = f"Test failure parsing file {filename} with format {fmt}"
         if comp:
             mode = "r" + self.get_mode(fmt)
             with gzip.open(filename, mode) as handle:
-                id_list = [rec.id for rec in SeqIO.parse(handle, fmt, alphabet)]
+                id_list = [rec.id for rec in SeqIO.parse(handle, fmt)]
         else:
-            id_list = [rec.id for rec in SeqIO.parse(filename, fmt, alphabet)]
+            id_list = [rec.id for rec in SeqIO.parse(filename, fmt)]
 
         with warnings.catch_warnings():
-            if "_alt_index_" in filename:
+            if "_alt_index_" in str(filename):
                 # BiopythonParserWarning: Could not parse the SFF index:
                 # Unknown magic number b'.diy' in SFF index header:
                 # b'.diy1.00'
                 warnings.simplefilter("ignore", BiopythonParserWarning)
 
-            rec_dict = SeqIO.index(filename, fmt, alphabet)
+            rec_dict = SeqIO.index(filename, fmt)
             self.check_dict_methods(rec_dict, id_list, id_list, msg=msg)
             rec_dict.close()
 
@@ -459,7 +458,7 @@ class IndexDictTests(SeqIOTestBaseClass):
 
             # In memory,
             # note here give filenames as list of strings
-            rec_dict = SeqIO.index_db(":memory:", [filename], fmt, alphabet)
+            rec_dict = SeqIO.index_db(":memory:", [filename], fmt)
             self.check_dict_methods(rec_dict, id_list, id_list, msg=msg)
             rec_dict.close()
 
@@ -477,13 +476,13 @@ class IndexDictTests(SeqIOTestBaseClass):
             # To disk,
             # note here we give the filename as a single string
             # to confirm that works too.
-            rec_dict = SeqIO.index_db(index_tmp, filename, fmt, alphabet)
+            rec_dict = SeqIO.index_db(index_tmp, filename, fmt)
             self.check_dict_methods(rec_dict, id_list, id_list, msg=msg)
             rec_dict.close()
             rec_dict._con.close()  # hack for PyPy
 
             # Now reload it...
-            rec_dict = SeqIO.index_db(index_tmp, [filename], fmt, alphabet)
+            rec_dict = SeqIO.index_db(index_tmp, [filename], fmt)
             self.check_dict_methods(rec_dict, id_list, id_list, msg=msg)
             rec_dict.close()
             rec_dict._con.close()  # hack for PyPy
@@ -493,7 +492,7 @@ class IndexDictTests(SeqIOTestBaseClass):
             index_tmp = os.path.abspath(index_tmp)
             os.chdir(os.path.dirname(filename))
             try:
-                rec_dict = SeqIO.index_db(index_tmp, alphabet=alphabet)
+                rec_dict = SeqIO.index_db(index_tmp)
             finally:
                 os.chdir(CUR_DIR)
             self.check_dict_methods(rec_dict, id_list, id_list, msg=msg)
@@ -506,26 +505,26 @@ class IndexDictTests(SeqIOTestBaseClass):
         """Sample key_function for testing index code."""
         return "id_" + key
 
-    def key_check(self, filename, fmt, alphabet, comp):
+    def key_check(self, filename, fmt, comp):
         """Check indexing with a key function."""
-        msg = "Test failure parsing file %s with format %s" % (filename, fmt)
+        msg = f"Test failure parsing file {filename} with format {fmt}"
         if comp:
             mode = "r" + self.get_mode(fmt)
             with gzip.open(filename, mode) as handle:
-                id_list = [rec.id for rec in SeqIO.parse(handle, fmt, alphabet)]
+                id_list = [rec.id for rec in SeqIO.parse(handle, fmt)]
         else:
-            id_list = [rec.id for rec in SeqIO.parse(filename, fmt, alphabet)]
+            id_list = [rec.id for rec in SeqIO.parse(filename, fmt)]
 
         key_list = [self.add_prefix(id) for id in id_list]
 
         with warnings.catch_warnings():
-            if "_alt_index_" in filename:
+            if "_alt_index_" in str(filename):
                 # BiopythonParserWarning: Could not parse the SFF index:
                 # Unknown magic number b'.diy' in SFF index header:
                 # b'.diy1.00'
                 warnings.simplefilter("ignore", BiopythonParserWarning)
 
-            rec_dict = SeqIO.index(filename, fmt, alphabet, self.add_prefix)
+            rec_dict = SeqIO.index(filename, fmt, key_function=self.add_prefix)
             self.check_dict_methods(rec_dict, key_list, id_list, msg=msg)
             rec_dict.close()
 
@@ -534,22 +533,29 @@ class IndexDictTests(SeqIOTestBaseClass):
 
             # In memory,
             rec_dict = SeqIO.index_db(
-                ":memory:", [filename], fmt, alphabet, self.add_prefix
+                ":memory:", [filename], fmt, key_function=self.add_prefix
             )
             self.check_dict_methods(rec_dict, key_list, id_list, msg=msg)
             # check error conditions
             with self.assertRaises(ValueError, msg=msg):
                 SeqIO.index_db(":memory:", format="dummy", key_function=self.add_prefix)
             with self.assertRaises(ValueError, msg=msg):
-                SeqIO.index_db(":memory:", filenames=["dummy"], key_function=self.add_prefix)
+                SeqIO.index_db(
+                    ":memory:", filenames=["dummy"], key_function=self.add_prefix
+                )
             rec_dict.close()
 
             # Saving to file...
-            index_tmp = filename + ".key.idx"
-            if os.path.isfile(index_tmp):
-                os.remove(index_tmp)
+            index_tmp = None
+            if isinstance(filename, str):
+                index_tmp = filename + ".key.idx"
+            elif isinstance(filename, Path):
+                index_tmp = filename.with_suffix(".key.idx")
+
+            if os.path.isfile(str(index_tmp)):
+                os.remove(str(index_tmp))
             rec_dict = SeqIO.index_db(
-                index_tmp, [filename], fmt, alphabet, self.add_prefix
+                index_tmp, [filename], fmt, key_function=self.add_prefix
             )
             self.check_dict_methods(rec_dict, key_list, id_list, msg=msg)
             rec_dict.close()
@@ -557,69 +563,52 @@ class IndexDictTests(SeqIOTestBaseClass):
 
             # Now reload it...
             rec_dict = SeqIO.index_db(
-                index_tmp, [filename], fmt, alphabet, self.add_prefix
+                index_tmp, [filename], fmt, key_function=self.add_prefix
             )
             self.check_dict_methods(rec_dict, key_list, id_list, msg=msg)
             rec_dict.close()
             rec_dict._con.close()  # hack for PyPy
 
             # Now reload without passing filenames and format
-            rec_dict = SeqIO.index_db(
-                index_tmp, alphabet=alphabet, key_function=self.add_prefix
-            )
+            rec_dict = SeqIO.index_db(index_tmp, key_function=self.add_prefix)
             self.check_dict_methods(rec_dict, key_list, id_list, msg=msg)
             rec_dict.close()
             rec_dict._con.close()  # hack for PyPy
-            os.remove(index_tmp)
+            os.remove(str(index_tmp))
             # Done
 
-    def get_raw_check(self, filename, fmt, alphabet, comp):
+    def get_raw_check(self, filename, fmt, comp):
         # Also checking the key_function here
-        msg = "Test failure parsing file %s with format %s" % (filename, fmt)
+        msg = f"Test failure parsing file {filename} with format {fmt}"
         if comp:
             with gzip.open(filename, "rb") as handle:
                 raw_file = handle.read()
             mode = "r" + self.get_mode(fmt)
             with gzip.open(filename, mode) as handle:
-                id_list = [
-                    rec.id.lower() for rec in SeqIO.parse(handle, fmt, alphabet)
-                ]
+                id_list = [rec.id.lower() for rec in SeqIO.parse(handle, fmt)]
         else:
             with open(filename, "rb") as handle:
                 raw_file = handle.read()
-            id_list = [
-                rec.id.lower() for rec in SeqIO.parse(filename, fmt, alphabet)
-            ]
+            id_list = [rec.id.lower() for rec in SeqIO.parse(filename, fmt)]
 
         if fmt in ["sff"]:
             with warnings.catch_warnings():
                 warnings.simplefilter("ignore", BiopythonParserWarning)
-                rec_dict = SeqIO.index(
-                    filename, fmt, alphabet, key_function=str.lower
-                )
+                rec_dict = SeqIO.index(filename, fmt, key_function=str.lower)
                 if sqlite3:
                     rec_dict_db = SeqIO.index_db(
-                        ":memory:",
-                        filename,
-                        fmt,
-                        alphabet,
-                        key_function=str.lower,
+                        ":memory:", filename, fmt, key_function=str.lower
                     )
         else:
-            rec_dict = SeqIO.index(filename, fmt, alphabet, key_function=str.lower)
+            rec_dict = SeqIO.index(filename, fmt, key_function=str.lower)
             if sqlite3:
                 rec_dict_db = SeqIO.index_db(
-                    ":memory:",
-                    filename,
-                    fmt,
-                    alphabet,
-                    key_function=str.lower,
+                    ":memory:", filename, fmt, key_function=str.lower
                 )
 
-        self.assertEqual(set(id_list), set(rec_dict), msg=msg)
+        self.assertCountEqual(id_list, rec_dict.keys(), msg=msg)
         if sqlite3:
-            self.assertEqual(set(id_list), set(rec_dict_db), msg=msg)
-        self.assertEqual(len(id_list), len(rec_dict), msg=msg)
+            self.assertCountEqual(id_list, rec_dict_db.keys(), msg=msg)
         for key in id_list:
             self.assertIn(key, rec_dict, msg=msg)
             self.assertEqual(key, rec_dict[key].id.lower(), msg=msg)
@@ -644,14 +633,13 @@ class IndexDictTests(SeqIOTestBaseClass):
             elif mode == "t":
                 handle = StringIO(raw.decode())
             else:
-                raise RuntimeError("Unexpected mode %s" % mode)
+                raise RuntimeError(f"Unexpected mode {mode}")
             if fmt == "sff":
                 rec2 = SeqIO.SffIO._sff_read_seq_record(
                     handle,
                     rec_dict._proxy._flows_per_read,
                     rec_dict._proxy._flow_chars,
                     rec_dict._proxy._key_sequence,
-                    rec_dict._proxy._alphabet,
                     trim=False,
                 )
             elif fmt == "sff-trim":
@@ -660,7 +648,6 @@ class IndexDictTests(SeqIOTestBaseClass):
                     rec_dict._proxy._flows_per_read,
                     rec_dict._proxy._flow_chars,
                     rec_dict._proxy._key_sequence,
-                    rec_dict._proxy._alphabet,
                     trim=True,
                 )
             elif fmt == "uniprot-xml":
@@ -680,13 +667,34 @@ class IndexDictTests(SeqIOTestBaseClass):
                     % raw.decode()
                 )
                 handle = StringIO(raw)
-                rec2 = SeqIO.read(handle, fmt, alphabet)
+                rec2 = SeqIO.read(handle, fmt)
             else:
-                rec2 = SeqIO.read(handle, fmt, alphabet)
-            self.assertEqual(True, compare_record(rec1, rec2))
+                rec2 = SeqIO.read(handle, fmt)
+            self.compare_record(rec1, rec2)
         rec_dict.close()
         del rec_dict
 
+    if sqlite3:
+
+        def test_alpha_fails_db(self):
+            """Reject alphabet argument in Bio.SeqIO.index_db()."""
+            # In historic usage, alphabet=... would be a Bio.Alphabet object.
+            self.assertRaises(
+                ValueError,
+                SeqIO.index_db,
+                ":memory:",
+                ["Fasta/dups.fasta"],
+                "fasta",
+                alphabet="XXX",
+            )
+
+    def test_alpha_fails(self):
+        """Reject alphabet argument in Bio.SeqIO.index()."""
+        # In historic usage, alphabet=... would be a Bio.Alphabet object.
+        self.assertRaises(
+            ValueError, SeqIO.index, "Fasta/dups.fasta", "fasta", alphabet="XXX"
+        )
+
     if sqlite3:
 
         def test_duplicates_index_db(self):
@@ -706,31 +714,58 @@ class IndexDictTests(SeqIOTestBaseClass):
             self.assertRaises(ValueError, SeqIO.to_dict, iterator)
 
     def test_simple_checks(self):
-        for filename1, fmt, alphabet in self.tests:
+        for filename1, fmt in self.tests:
             assert fmt in _FormatToRandomAccess
             tasks = [(filename1, None)]
             if os.path.isfile(filename1 + ".bgz"):
                 tasks.append((filename1 + ".bgz", "bgzf"))
             for filename2, comp in tasks:
-                self.simple_check(filename2, fmt, alphabet, comp)
+                self.simple_check(filename2, fmt, comp)
+
+    def test_simple_checks_with_pathobj(self):
+        for filename1, fmt in self.tests:
+            assert fmt in _FormatToRandomAccess
+            tasks = [(filename1, None)]
+            if os.path.isfile(filename1 + ".bgz"):
+                tasks.append((filename1 + ".bgz", "bgzf"))
+            for filename2, comp in tasks:
+                self.simple_check(Path(filename2), fmt, comp)
 
     def test_key_checks(self):
-        for filename1, fmt, alphabet in self.tests:
+        for filename1, fmt in self.tests:
             assert fmt in _FormatToRandomAccess
             tasks = [(filename1, None)]
             if os.path.isfile(filename1 + ".bgz"):
                 tasks.append((filename1 + ".bgz", "bgzf"))
             for filename2, comp in tasks:
-                self.key_check(filename2, fmt, alphabet, comp)
+                self.key_check(filename2, fmt, comp)
+
+    def test_key_checks_with_pathobj(self):
+        for filename1, fmt in self.tests:
+            assert fmt in _FormatToRandomAccess
+            tasks = [(filename1, None)]
+            if os.path.isfile(filename1 + ".bgz"):
+                tasks.append((filename1 + ".bgz", "bgzf"))
+            for filename2, comp in tasks:
+                self.key_check(Path(filename2), fmt, comp)
 
     def test_raw_checks(self):
-        for filename1, fmt, alphabet in self.tests:
+        for filename1, fmt in self.tests:
             assert fmt in _FormatToRandomAccess
             tasks = [(filename1, None)]
             if os.path.isfile(filename1 + ".bgz"):
                 tasks.append((filename1 + ".bgz", "bgzf"))
             for filename2, comp in tasks:
-                self.get_raw_check(filename2, fmt, alphabet, comp)
+                self.get_raw_check(filename2, fmt, comp)
+
+    def test_raw_checks_with_pathobj(self):
+        for filename1, fmt in self.tests:
+            assert fmt in _FormatToRandomAccess
+            tasks = [(filename1, None)]
+            if os.path.isfile(filename1 + ".bgz"):
+                tasks.append((filename1 + ".bgz", "bgzf"))
+            for filename2, comp in tasks:
+                self.get_raw_check(Path(filename2), fmt, comp)
 
 
 class IndexOrderingSingleFile(unittest.TestCase):
