@@ -20,13 +20,11 @@ except ImportError:
 
 import json
 import unittest
-
+import warnings
 from io import StringIO
 
 from Bio import BiopythonExperimentalWarning
 
-import warnings
-
 with warnings.catch_warnings():
     warnings.simplefilter("ignore", BiopythonExperimentalWarning)
     from Bio import phenotype
@@ -75,9 +73,7 @@ class TestPhenoMicro(unittest.TestCase):
             # I want to see the output when called from the test harness,
             # run_tests.py (which can be funny about new lines on Windows)
             handle.seek(0)
-            raise ValueError(
-                "%s\n\n%s\n\n%s" % (str(e), repr(handle.read()), repr(records))
-            ) from None
+            self.fail(f"{e}\n\n{handle.read()!r}\n\n{records!r}")
 
         self.assertEqual(p1, records[0])
 
@@ -132,8 +128,8 @@ class TestPhenoMicro(unittest.TestCase):
         p["A02"] = p["A02"]
         for w in p:
             pass
-        self.assertEqual("A01" in p, True)
-        self.assertEqual("test" in p, False)
+        self.assertIn("A01", p)
+        self.assertNotIn("test", p)
         self.assertRaises(ValueError, next, p.get_row("test"))
         self.assertEqual(next(p.get_row("A")), p["A01"])
         self.assertRaises(ValueError, next, p.get_column("test"))
@@ -256,13 +252,13 @@ class TestPhenoMicro(unittest.TestCase):
         )
 
         w.fit(None)
-        self.assertEqual(w.area, None)
-        self.assertEqual(w.model, None)
-        self.assertEqual(w.lag, None)
-        self.assertEqual(w.plateau, None)
-        self.assertEqual(w.slope, None)
-        self.assertEqual(w.v, None)
-        self.assertEqual(w.y0, None)
+        self.assertIsNone(w.area)
+        self.assertIsNone(w.model)
+        self.assertIsNone(w.lag)
+        self.assertIsNone(w.plateau)
+        self.assertIsNone(w.slope)
+        self.assertIsNone(w.v)
+        self.assertIsNone(w.y0)
         self.assertEqual(w.max, 313.0)
         self.assertEqual(w.min, 29.0)
         self.assertEqual(w.average_height, 217.82552083333334)
@@ -313,6 +309,24 @@ class TestPhenoMicro(unittest.TestCase):
 
         w[1] = 1
 
+    def test_JsonIterator(self):
+        """Test basic functionalities of JsonIterator file parser."""
+        # Parse file content big enough to trigger issue #3783
+        handle = StringIO(
+            '{"csv_data": {"Plate Type": "PM-999"}, "measurements": {"Hour": 9}}'
+        )
+        with self.assertWarnsRegex(UserWarning, "PM-999"):
+            for w in phenotype.phen_micro.JsonIterator(handle):
+                self.assertEqual(w.id, "PM999")
+
+    def test_CsvIterator(self):
+        """Test basic functionalities of CsvIterator file parser."""
+        # Parse file content big enough to trigger issue #3783
+        handle = StringIO('"Data File",3\n"Plate Type",PM-33\n')
+        with self.assertWarnsRegex(UserWarning, "PM-33"):
+            for w in phenotype.phen_micro.CsvIterator(handle):
+                self.assertEqual(w.id, "PM33")
+
 
 if __name__ == "__main__":
     runner = unittest.TextTestRunner(verbosity=2)
